"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _os = _interopRequireDefault(require("os"));

var _utils = require("../utils");

var _testPattern = require("./testPattern");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let isWindows = _os.default.platform().toLowerCase() === 'win32';
let ps = _path.default.sep;
let options = (0, _utils.getOptions)();
let {
  app: {
    vendorExclude,
    customChunks,
    vendorInclude
  }
} = options;

let isVendor = function isVendor(module) {
  let {
    userRequest
  } = module;
  let excludeList = ['script-loader', 'raw-loader', 'react', 'react-dom'];
  excludeList = [...excludeList, ...vendorExclude];
  return userRequest && (vendorInclude.some(item => userRequest.indexOf(item) !== -1) || userRequest.indexOf('node_modules') >= 0 && userRequest.endsWith('.css') === false && userRequest.endsWith('publicPathConfig.js') === false && excludeList.every(item => userRequest.indexOf(`node_modules${ps}${item}${ps}`) === -1));
};

let isReact = module => {
  let {
    userRequest
  } = module;
  let reactBundle = ['react', 'react-dom'];
  return userRequest && reactBundle.some(pkg => userRequest.indexOf(`node_modules${ps}${pkg}${ps}`) >= 0);
};

let defaultChunks = {
  'react.vendor': {
    name: 'react.vendor',
    chunks: 'all',
    minChunks: 1,
    test: isReact,
    enforce: true,
    // priority: 10
    priority: -10
  },
  vendor: {
    name: 'vendor',
    chunks: 'all',
    // chunks: 'initial',
    minChunks: 1,
    test: isVendor,
    // priority: 19,
    priority: -10
  }
};
let customChunksConfig = {};
customChunks.map((obj, index) => ({
  name: obj.name,
  pattern: obj.pattern,
  minChunks: obj.minChunks || obj.size || 2,
  rules: obj.rules,
  // includeDepenency: obj.includeDepenency || false,
  priority: obj.priority || -10 * (index + 2),
  enforce: obj.enforce || true,
  maxSize: obj.maxSize,
  // || 0,
  minSize: obj.minSize,
  // || 20000,
  chunks: obj.chunks || 'all'
})).map(({
  name,
  pattern,
  minChunks,
  rules,
  priority,
  // includeDepenency,
  enforce,
  minSize,
  maxSize,
  chunks = 'all'
}) => {
  let obj = {
    name,
    test: rules ? m => {
      const {
        userRequest
      } = m;
      return (0, _testPattern.testPattern)(userRequest, rules); // return (
      //   pkgs.some(p => isRelated(userRequest, p)) ||
      //   (includeDepenency && isDependency(m, pkgs))
      // );
    } : new RegExp(isWindows ? pattern.replace(/\//g, '\\') : pattern),
    chunks,
    enforce,
    minChunks,
    priority
  };

  if (minSize !== undefined) {
    obj.minSize = minSize;
  }

  if (maxSize !== undefined) {
    obj.maxSize = maxSize;
  }

  return customChunksConfig[name] = obj;
});
var _default = {
  minSize: 12000,
  cacheGroups: Object.assign({
    default: false,
    vendors: false
  }, defaultChunks, customChunksConfig)
};
exports.default = _default;