"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _miniCssExtractPlugin = _interopRequireDefault(require("mini-css-extract-plugin"));

var _cssClassNameGenerate = _interopRequireDefault(require("../utils/cssClassNameGenerate"));

var _utils = require("../utils");

var _windowsModification = require("./windowsModification");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const fs = require('fs');

const options = (0, _utils.getOptions)();

function excludeEmptyCheckPlugin({
  enable,
  ignore,
  plugins
}) {
  return enable ? ignore.length === 0 ? plugins : [require('../postcss-plugins/ExcludePlugin')({
    ignore,
    plugins
  })] : [];
}

const getCSSLoaders = optionsObj => {
  const {
    plugins,
    exclude,
    mediaQueryHoverActiveString,
    cssVariableReplacementConfig,
    classNameBlob,
    cssUniqueness,
    selectorReplace,
    cssHashSelectors,
    classNamePrefix
  } = optionsObj; // console.log('plugins:')
  // console.log(plugins)
  // console.log('exclude:')
  // console.log(exclude)

  const {
    devCssFileBountry
  } = options.app;
  const {
    valueReplacer
  } = options.css;
  const {
    cssSelectorZipPath
  } = options.impactService;
  const rtlExcludeLocal = (0, _windowsModification.windowsModification)(exclude.rtl);
  const hoverActiveExcludeLocal = (0, _windowsModification.windowsModification)(exclude.hoverActive);
  const combinerMediaQueryExcludeLocal = (0, _windowsModification.windowsModification)(exclude.combinerMediaQuery);
  const cssVariableReplacementExcludeLocal = (0, _windowsModification.windowsModification)(exclude.cssVariableReplacement);
  const cssLoaderOptions = {
    // importLoaders: hasRTL||hoverActive ? 1 : 0,
    importLoaders: 1,
    modules: {},
    sourceMap: true
  };

  if (classNameBlob) {
    cssLoaderOptions.modules.localIdentName = classNameBlob;
  } else {
    cssLoaderOptions.modules.getLocalIdent = (0, _cssClassNameGenerate.default)(cssUniqueness, cssHashSelectors, classNamePrefix);
  } // console.log('selector weight config : ', selectorWeightConfig);


  const postcssPlugins = [valueReplacer && require('../postcss-plugins/ValueReplacer')(valueReplacer), selectorReplace && require('postcss-selector-replace')(selectorReplace), ...excludeEmptyCheckPlugin({
    enable: plugins.hasRTL,
    ignore: rtlExcludeLocal,
    plugins: [require('@zohodesk/postcss-rtl')({
      addPrefixToSelector: function addPrefixToSelector(selector, prefix) {
        if (prefix === '[dir]') {
          return selector;
        }

        return `${prefix} ${selector}`; // Make selectors like [dir=rtl] > .selector
      }
    })]
  }), ...excludeEmptyCheckPlugin({
    enable: plugins.combinerMediaQuery,
    ignore: combinerMediaQueryExcludeLocal,
    plugins: [require('postcss-combine-media-query')]
  }), ...excludeEmptyCheckPlugin({
    enable: plugins.hoverActive,
    ignore: hoverActiveExcludeLocal,
    plugins: [require('../postcss-plugins/hoverActivePlugin')(mediaQueryHoverActiveString)]
  }), ...excludeEmptyCheckPlugin({
    enable: plugins.cssVariableReplacement,
    ignore: cssVariableReplacementExcludeLocal,
    plugins: [fs.existsSync(cssVariableReplacementConfig) && require('../postcss-plugins/variableModificationPlugin/index').plugin(cssVariableReplacementConfig)]
  })].filter(Boolean);
  return [cssSelectorZipPath && {
    loader: require.resolve('../loaders/selectorMappingLoader')
  }, {
    loader: _miniCssExtractPlugin.default.loader,
    options: {
      publicPath: '../'
    }
  }, {
    loader: 'css-loader',
    options: cssLoaderOptions
  }, devCssFileBountry && {
    loader: require.resolve('../loaders/fileBountryLoader'),
    options: {
      devCssFileBountry
    }
  }, postcssPlugins.length > 0 ? {
    loader: 'postcss-loader',
    options: {
      ident: 'postcss',
      plugins: function () {
        return postcssPlugins;
      }
    }
  } : null].filter(Boolean);
};

var _default = getCSSLoaders;
exports.default = _default;