"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _postcss = _interopRequireDefault(require("postcss"));

var _fs = _interopRequireDefault(require("fs"));

var _webpackSources = require("webpack-sources");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const path = require('path');

const isCss = filename => path.extname(filename) === '.css';

function rootConvertor(rootOriginal, defaultSelector, data, chunkFilename, excludeStrings) {
  rootOriginal.walkRules((rule, index) => {
    const prevNode = rootOriginal.nodes[index - 1];

    if (index !== 0 && prevNode.type && prevNode.type === 'comment' && prevNode.text.toLowerCase() === 'updateselector:ignore') {
      return;
    }

    if (rule.parent && rule.parent.type === 'atrule') {
      if (rule.parent.name !== 'media') {
        return;
      }
    }

    if (excludeStrings.includes(rule.selector)) {
      return;
    }

    let ruleArr = rule.selector.split(',');
    ruleArr.forEach((rule, index) => {
      if (rule.includes(']')) {
        if (rule.slice(rule.lastIndexOf(']') + 2).trim() === '') {
          return;
        } // console.log(
        //   rule.slice(rule.lastIndexOf(']') + 2).trim(),
        //   rule.slice(rule.lastIndexOf(']') + 2).trim() !== ''
        // );


        ruleArr[index] = `${rule.slice(0, rule.lastIndexOf(']') + 1).trim()} ${defaultSelector}${rule.slice(rule.lastIndexOf(']') + 2).trim()}`; // console.log(ruleArr[index]);

        return;
      }

      ruleArr[index] = data[chunkFilename] ? `${data[chunkFilename].defaultSelector}${rule}` : `${defaultSelector}${rule}`;
    });
    rule.selector = ruleArr.join(',\n'); // console.log(`${rule.selector} converted`);
  });
  return rootOriginal;
}

class SelectorPlugin {
  constructor(options = {}) {
    this.optimize = options.optimize;
    this.selectorWeightConfig = options.selectorWeightConfig;
  }

  apply(compiler) {
    const {
      defaultSelector,
      customFileDetails,
      excludeStrings
    } = this.selectorWeightConfig;

    const rawdata = _fs.default.readFileSync(customFileDetails);

    const data = JSON.parse(rawdata);
    compiler.hooks.compilation.tap('selector-weight-rewrite', compilation => {
      compilation.hooks.optimizeChunkAssets.tapAsync('selector-weight-rewrite', (chunks, callback) => {
        chunks.forEach(chunk => {
          chunk.files.filter(isCss).forEach(chunkFilename => {
            const asset = compilation.assets[chunkFilename];
            const sourceStr = asset.source();

            const rootOriginal = _postcss.default.parse(sourceStr);

            compilation.assets[chunkFilename] = new _webpackSources.RawSource(rootConvertor(rootOriginal, defaultSelector, data, chunkFilename, excludeStrings).toString());
          });
        });
        callback();
      });
    });
  }

}

var _default = SelectorPlugin;
exports.default = _default;