"use strict";

const postcss = require('postcss');

const path = require('path');

const fs = require('fs');

function populateArray(start, end) {
  let temp = [];

  for (var i = start; i < end; i++) {
    temp.push(i);
  }

  return temp;
}

let allwdVars = {
  'font-size': ['px', 'em']
};
let numberObject = {
  'font-size': {
    allowed: ['px', 'em'],
    replacements: {
      px: 'var(--zd_font_size$$)',
      em: 'var(--zd_font_size$$em)'
    },
    //[5,10,15,20,25],
    available: populateArray(0, 251),
    replacementValues: {
      px: [],
      em: []
    }
  },
  'margin': {
    allowed: ['px'],
    replacements: {
      px: 'var(--zd_size$$)'
    },
    available: populateArray(-250, 251),
    replacementValues: {
      px: []
    }
  },
  'margin-left': {
    allowed: ['px'],
    replacements: {
      px: 'var(--zd_size$$)'
    },
    available: populateArray(-250, 251),
    replacementValues: {
      px: []
    }
  },
  'margin-right': {
    allowed: ['px'],
    replacements: {
      px: 'var(--zd_size$$)'
    },
    available: populateArray(-250, 251),
    replacementValues: {
      px: []
    }
  },
  'margin-top': {
    allowed: ['px'],
    replacements: {
      px: 'var(--zd_size$$)'
    },
    available: populateArray(-250, 251),
    replacementValues: {
      px: []
    }
  },
  'margin-bottom': {
    allowed: ['px'],
    replacements: {
      px: 'var(--zd_size$$)'
    },
    available: populateArray(-250, 251),
    replacementValues: {
      px: []
    }
  }
};

function hasIgnoreComment(node) {
  return node ? node.type == 'comment' ? node.text == 'Variable:Ignore' ? true : false : false : false;
}

let errors = [];
module.exports = postcss.plugin('postcss-variable-report', () => rootOriginal => {
  // console.log('inside postcss plugin')
  // fs.writeFile('./css_error.log', '\r\nLog File: \r\n\r\n', (err) => {
  //    if(err)console.log(err);
  // });
  // console.log(rootOriginal.source.input.file);
  if (!rootOriginal.source.input.file.includes('css_error')) {
    rootOriginal.walkRules(rule => {
      rule.walkDecls((decl, position) => {
        // case font-size
        if (!hasIgnoreComment(rule.nodes[position - 1])) {
          console.log(settings);
          let unit = decl.value.replace(/[0-9]/g, '');
          let settings = numberObject[decl.prop];
          let path = rootOriginal.source.input.from;
          let filename = path.split('\\');
          filename = filename[filename.length - 1];

          if (decl.prop === 'font-size' || decl.prop === 'margin-left' || decl.prop === 'margin-right' || decl.prop === 'margin-top' || decl.prop === 'margin-bottom') {
            let {
              allowed,
              replacements,
              available
            } = settings;

            if (!rootOriginal.source.input.from.includes('node_modules')) {
              if (unit != 0) {
                if (allowed.includes(unit)) {
                  // console.log(available, parseInt(decl.value))
                  if (available.includes(parseInt(decl.value))) {
                    // replacementValues[unit].push({[decl.value] : replacements[unit].replace('$$', parseInt(decl.value))})
                    decl.value = replacements[unit].replace('$$', parseInt(decl.value)); //console.log("replacements:")
                  } else {
                    let err = {
                      prop: decl.prop,
                      value: decl.value,
                      filename,
                      filepath: rootOriginal.source.input.from,
                      line: decl.source.start.line,
                      unit,
                      message: 'value not available consider others'
                    };
                    errors.push(err);
                  }
                } else {
                  let err = {
                    prop: decl.prop,
                    value: decl.value,
                    filename,
                    filepath: rootOriginal.source.input.from,
                    line: decl.source.start.line,
                    unit,
                    message: 'Unit not supported'
                  };
                  errors.push(err);
                }
              }
            }
          } else if (decl.prop === 'margin') {
            let {
              allowed,
              replacements,
              available
            } = settings; //console.log(decl.prop + "  " + decl.value)

            let valArr = decl.value.split(' '); //console.log(valArr)
            //console.log(allowed, replacements, available)

            let hasError = false;
            let newVal = '';
            valArr.forEach(val => {
              let unit = val.replace(parseInt(val), '');
              let numVal = parseInt(val);

              if (unit != 0) {
                if (allowed.includes(unit)) {
                  if (available.includes(numVal)) {
                    if (numVal >= 0) {
                      if (!hasError) {
                        newVal += replacements[unit].replace('$$', numVal) + " ";
                      }
                    } else {
                      if (!hasError) {
                        newVal += `calc(${replacements[unit].replace('$$', numVal * -1)} * -1)` + " ";
                      }
                    }
                  } else {
                    hasError = true;
                    let err = {
                      prop: decl.prop,
                      value: numVal,
                      filename,
                      filepath: rootOriginal.source.input.from,
                      line: decl.source.start.line,
                      unit,
                      message: 'value not available consider others'
                    };
                    errors.push(err);
                  }
                } else {
                  hasError = true;
                  let err = {
                    prop: decl.prop,
                    value: decl.value,
                    filename,
                    filepath: rootOriginal.source.input.from,
                    line: decl.source.start.line,
                    unit,
                    message: 'Unit not supported'
                  };
                  errors.push(err);
                }
              }
            });

            if (!hasError) {
              decl.value = newVal;
            }
          }
        }
      });
    }); //fs.writeFile('./src/trial.json', JSON.stringify(variables) ,(err) =>{
    //    if(err)console.log(err);
    //    else{
    //        console.log('updated successfully');
    //    }
    // })
    // console.log(errors)

    let errorArr = [];
    errors.forEach(errStr => {
      errorArr.push(` prop: ${errStr.prop} ,\n value : ${errStr.value} ,\n filename : ${errStr.filename} ,\n filepath : ${errStr.filepath} ,\n line : ${errStr.line} ,\n unit : ${errStr.unit} ,\n message : ${errStr.message} \r`);
    }); //fs.writeFile('./css_error.log','', (err) =>{
    //    if(err){
    //        console.log(err);
    //    }
    //});
    // errorArr.forEach(err => {
    //   let data = `\n{\n${err}\n},\n`;
    //   //fs.appendFile('./css_error.log',data,(err)=>{
    //   //    if(err){
    //   //       console.log(err);
    //   //    }
    //   //})
    // });
    // fs.writeFileSync('./css_error.log','');
    //  errorArr.forEach(err => {
    //    let data = `\n{\n${err}\n},\n`;
    //    fs.appendFileSync('./css_error.log',data);
    //  });
  }
});